#define NCP_KERNEL
#define NCP_LINUX
#define PROC_ARMA15
#include <ncp_eth_vp.h>

/* one time include */
#include <ncp_eth_vp.c>

#include "emtimer.h"

static ncp_hdl_t devHdl;
static ncp_task_hdl_t taskHdl;
static ncp_vp_hdl_t vpCpuCpu;
static DEFINE_RAW_SPINLOCK(timer_send_lock);

int __init em_send_init(void)
{
   ncp_st_t nerr;

   nerr = ncp_dev_open(0, 0, &devHdl);
   if (nerr) {
      pr_err("emtimer: ncp_dev_open err %d %s\n", nerr, ncp_status_get(nerr));
      goto out;
   }

   //FIXME?
   //we may need to wait for config load to do this
   nerr = ncp_task_hdl_create(devHdl, NCP_NCA_QG_15_TQS_TimerTQS,
      FALSE /* shared - we do our own locking */,
      FALSE /* ordered */,
      &taskHdl);
   if (nerr) {
      pr_err("emtimer: ncp_task_hdl_create err %d %s\n", nerr, ncp_status_get(nerr));
      goto out_dev;
   }
   
   /* REALLY disable shared mode! */
   nerr = ncp_task_send_mutex_disable(taskHdl);
   if (nerr) {
      pr_err("emtimer: ncp_task_send_mutex_disable err %d %s\n", nerr, ncp_status_get(nerr));
      goto out_hdl;
   }
   
   //TODO
   //do we need these as well?
   //i don't think we alloc
   //ncp_task_ncav2_use_exclusive_allocator(&taskHdl);?
   //ncp_task_ncav2_cpuPool_mutex_disable?

   nerr = ncp_vp_handle_get_cpu_em_cpu_cpu(devHdl, &vpCpuCpu);
   if (nerr) {
      pr_err("emtimer: ncp_vp_handle_get_cpu_em_cpu_cpu err %d %s\n", nerr, ncp_status_get(nerr));
      goto out_hdl;
   }
   
   return 0;

out_hdl:
   ncp_task_hdl_remove(taskHdl);
out_dev:
   ncp_dev_close(devHdl);
out:
   return -ENODEV;
}

void __exit em_send_exit(void)
{
   ncp_task_hdl_remove(taskHdl);

   ncp_dev_close(devHdl);
}

int em_send(void *event, u32 queue)
{
   ncp_task_send_meta_t meta;
   ncp_vp_send_em_cpu_cpu_t *params = (void*)meta.taskParms;
   unsigned long flags;
   u8 did;
   ncp_st_t nerr;
   
   did = (u8)(queue>>24);
   queue &= ((1<<24)-1);

   pr_info("em_send(%08x, %04x, %p)\n", did, queue, event);
   
   memset(&meta, 0x00, sizeof(meta));
   
   meta.priority = 1;

   params->emLocalQueueID = cpu_to_be32(queue);
   params->type = __constant_cpu_to_be32(2 << 24); //TIMER
   params->pEvent = cpu_to_be32((u32)event);
   params->DID = did;
   //params->fromCore = smp_processor_id();
   
   /* we need to lock outside the API to protect against concurrent sends to this TQS */
   raw_spin_lock_irqsave(&timer_send_lock, flags);
   nerr = ncp_task_send(taskHdl, vpCpuCpu, &meta,
      NULL /* buf */, NULL /* pdu */, FALSE /* nonblocking */,
      NULL /* callback */, NULL /* cb arg */);
   raw_spin_unlock_irqrestore(&timer_send_lock, flags);
   
   if (nerr) {
      pr_err("emtimer: em_send err %d %s\n", nerr, ncp_status_get(nerr));
      return -ECOMM;
   }
   
   return 0;
}
