/*
 * Copyright (c) 2011-2014 Nokia Solutions and Networks
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/**
 * @file fzm_ioctls.h
 * @author Nokia Solutions and Networks
 * @remarks Copyright (C) 2011-2014 Nokia Solutions and Networks
 *
 * @par PURPOSE IOCTL definitions for all of the kernel modules for FZM.
 *
 * @par REVISIONS
 * <pre>
 * Date       CoreID    CR         Description
 * ---------- --------- ---------- --------------------------------------------
 * 07/27/2011 NSN                  Initial Creation
 * 01/31/2012 tberger1  1458932    Updates for krypton
 * 04/11/2013 bbausch1             Porting to FZM
 * 09/09/2013 nismail1             added sfp ioctls
 * 10/09/2014 samethad             adding marvel_sw ioctls
 * 10/17/2014 bwheele1             adding fpga_frame ioctls
 * 11/20/2014 bwheele1             adding em_wait ioctls
 * 12/11/2014 kryan1               adding fpga_smr ioctl
 * </pre>
 */

#ifndef FZM_IOCTLS_H
#define FZM_IOCTLS_H

#include <linux/ioctl.h>

#define FZM_IOCTL_ID     0xE5    // random unused ioctl id

/*
 * _IO    an ioctl with no parameters
 * _IOW   an ioctl with write parameters (copy_from_user)
 * _IOR   an ioctl with read parameters  (copy_to_user)
 * _IOWR  an ioctl with both write and read parameters.
 */

/*
 * Name: cpld-usr
 * Allocated IOCTL range: 0x00 - 0x0F (16)
 */
#define CPLD_RESET_DEVICE           _IOW (FZM_IOCTL_ID, 0x00, Cpld_device_t)        /**< Put a device into reset. */
#define CPLD_RELEASE_RESET_DEVICE   _IOW (FZM_IOCTL_ID, 0x01, Cpld_device_t)        /**< Take a device out of reset. */
#define CPLD_RD_RESET_STATUS        _IOR (FZM_IOCTL_ID, 0x02, uint8_t)              /**< Read RRR2 to obtain the reset status of a device */
#define CPLD_SET_LED                _IOW (FZM_IOCTL_ID, 0x03, struct cpld_led)      /**< Set LED status */
#define CPLD_GET_LED                _IOWR(FZM_IOCTL_ID, 0x04, struct cpld_led)      /**< Read LED status */
#define CPLD_SET_SCR5               _IOW (FZM_IOCTL_ID, 0x07, uint8_t)              /**< Set scratchpad register 5 */
#define CPLD_GET_SCR5               _IOWR(FZM_IOCTL_ID, 0x08, uint8_t)              /**< Get scratchpad register 5 */
#define CPLD_TOGGLE_SCR5            _IOW (FZM_IOCTL_ID, 0x09, struct cpld_toggle_scr5) /**< Toggle bits in scratchpad register 5 */
#define CPLD_GET_OFFSET             _IOR (FZM_IOCTL_ID, 0x0A, struct cpld_ioctl_rw)      /**< Read given cpld reg */
#define CPLD_SET_OFFSET             _IOW (FZM_IOCTL_ID, 0x0B, struct cpld_ioctl_rw)      /**< write given cpld reg */ 

/*
 * Name: cpld-sfp-usr
 * Allocated IOCTL range: 0x10 - 0x1F (16)
 */
#define CPLD_SFP_SWSTAT_READ              _IOWR(FZM_IOCTL_ID, 0x10, uint8_t )       /**< Read swstatus register */

#define CPLD_SFP_CTRL_READ		_IOWR(FZM_IOCTL_ID, 0x11, struct cpld_sfpc_read)       /**< Read SFP control register */

#define CPLD_SFP_CTRL_WRITE		_IOWR(FZM_IOCTL_ID, 0x12, struct cpld_sfpc_write)       /**< Write SFP Control register */

#define CPLD_SFP_SERIAL_READ            _IOWR(FZM_IOCTL_ID, 0x13, struct cpld_sfp_serial_read)       /**< Read SFP serial register */
#define CPLD_SFP_SERIAL_WRITE           _IOWR (FZM_IOCTL_ID, 0x14, struct cpld_sfp_serial_write)       /**< Write SFP serial register */
#define CPLD_SFP_DIAG_READ              _IOW (FZM_IOCTL_ID, 0x15, struct cpld_sfp_diag_read)   /**< Read diag register bits */
#define CPLD_SFP_DIAG_WRITE             _IOWR (FZM_IOCTL_ID, 0x16, struct cpld_sfp_diag_write)   /**< Write diag register bits */
#define CPLD_SFP_SWINTS                _IOWR(FZM_IOCTL_ID, 0x17, uint8_t )       /**< Read sw interrupt register */

#if 0
/*
 * Name: fpga-usr
 * Allocated IOCTL range: 0x10 - 0x1F (16)
 */
#define FPGA_READ                   _IOWR(FZM_IOCTL_ID, 0x10, struct fpga_rw)       /**< Read FPGA register */
#define FPGA_WRITE                  _IOW (FZM_IOCTL_ID, 0x11, struct fpga_rw)       /**< Write FPGA register */
#define FPGA_TOGGLE_BITS            _IOW (FZM_IOCTL_ID, 0x12, struct fpga_toggle)   /**< Toggle FPGA register bits */
#endif

/*
 * Name: rfup-usr
 * Allocated IOCTL range: 0x20 - 0x2F (16)
 */
#define RFUP_MESSAGE                _IOWR(FZM_IOCTL_ID, 0x20, struct rfup_ioctl_msg) /**< Send and receive uP message */

/*
 * Name: marvel-sw
 * Allocated IOCTL range: 0x30 - 0x4F (32)
 */
#define MARVEL_PHY_SET_AUTONEG            _IOWR(FZM_IOCTL_ID, 0x30, struct marswphy_ops)
#define MARVEL_PHY_GET_AUTONEG            _IOWR(FZM_IOCTL_ID, 0x31, struct marswphy_ops)
#define MARVEL_PHY_SET_DUPLEX_MODE        _IOWR(FZM_IOCTL_ID, 0x32, struct marswphy_ops)
#define MARVEL_PHY_GET_DUPLEX_MODE        _IOWR(FZM_IOCTL_ID, 0x33, struct marswphy_ops)
#define MARVEL_PHY_SET_COPPER_LOOPBACK    _IOWR(FZM_IOCTL_ID, 0x34, struct marswphy_ops)
#define MARVEL_PHY_GET_COPPER_LOOPBACK    _IOWR(FZM_IOCTL_ID, 0x35, struct marswphy_ops)
#define MARVEL_PHY_COPPER_RESTART_AUTONEG _IOWR(FZM_IOCTL_ID, 0x36, struct marswphy_ops)
#define MARVEL_PHY_COPPER_RESET           _IOWR(FZM_IOCTL_ID, 0x37, struct marswphy_ops)
#define MARVEL_PHY_COPPER_POWER           _IOWR(FZM_IOCTL_ID, 0x38, struct marswphy_ops)
#define MARVEL_PHY_SET_SPEED              _IOWR(FZM_IOCTL_ID, 0x39, struct marswphy_ops)
#define MARVEL_PHY_GET_SPEED              _IOWR(FZM_IOCTL_ID, 0x3A, struct marswphy_ops)
#define MARVEL_SWITCH_REGISTER_READ       _IOWR(FZM_IOCTL_ID, 0x3B, struct marswgeneric_ops)
#define MARVEL_SWITCH_REGISTER_WRITE      _IOWR(FZM_IOCTL_ID, 0x3C, struct marswgeneric_ops)
#define MARVEL_SWITCH_BIT_WRITE_TO_REG    _IOWR(FZM_IOCTL_ID, 0x3D, struct marswgeneric_ops)
#define MARVEL_SWITCH_SET_PORT_STATE      _IOWR(FZM_IOCTL_ID, 0x3E, struct marswport_ops)
#define MARVEL_SWITCH_GET_PORT_STATE      _IOWR(FZM_IOCTL_ID, 0x3F, struct marswport_ops)
#define MARVEL_STATS_COLLECTION_REQ       _IOWR(FZM_IOCTL_ID, 0x40, struct mvstats_ops)
#define MARVEL_STATS_READ_CAPTURED_COUNTERS _IOWR(FZM_IOCTL_ID, 0x41, struct mvstats_ops)
#define MARVEL_EGRESS_RATE_CONTROL        _IOWR(FZM_IOCTL_ID, 0x42, struct egress_rate_ops)
#define MARVEL_RESET_SWITCH               _IO(FZM_IOCTL_ID, 0x43)
#define MARVEL_STATS_OP_CONTROL          _IOWR(FZM_IOCTL_ID, 0x44, struct stats_int_ops)
#define MARVEL_PHY_SET_CONFIG_ADVERT      _IOWR(FZM_IOCTL_ID, 0x45, struct marswadvert_ops)
#define MARVEL_PHY_GET_CONFIG_ADVERT      _IOWR(FZM_IOCTL_ID, 0x46, struct marswadvert_ops)
#define MARVEL_PHY_SET_CROSSOVER_MODE     _IOWR(FZM_IOCTL_ID, 0x47, struct marswphy_ops)
#define MARVEL_PHY_GET_CROSSOVER_MODE     _IOWR(FZM_IOCTL_ID, 0x48, struct marswphy_ops)
#define MARVEL_PHY_SET_CONFIG_ADVERT_PAUSE _IOWR(FZM_IOCTL_ID, 0x49, struct marswphy_ops)
#define MARVEL_PHY_GET_CONFIG_ADVERT_PAUSE _IOWR(FZM_IOCTL_ID, 0x4A, struct marswphy_ops)
#define MARVEL_PHY_COPPER_STATUS          _IOWR(FZM_IOCTL_ID, 0x4B, struct marswstatus_ops)
#define MARVEL_INGRESS_RATE_CONTROL       _IOWR(FZM_IOCTL_ID, 0x4C, struct pirl_resource_info)

/*
 * Name: fpga_smr
 * Allocated IOCTL range: 0x50 - 0x5F (16)
 */
#define FPGA_SMR_IOCTL_CMD_WAIT_SMR       _IOR(FZM_IOCTL_ID,0x50,uint32_t)

/*
 * Name: fpga-frame
 * Allocated IOCTL range: 0x60 - 0x6F (16)
 */
#define FPGA_FRAME_WAIT_NEXT                _IO(FZM_IOCTL_ID, 0x60) /*wait for next frame*/
#define FPGA_FRAME_WAIT_FOR                 _IOW(FZM_IOCTL_ID, 0x61, struct fpga_frame_ioctl_wait_for) /*wait for indicated frame*/

/*
 * Name: em-wait
 * Allocated IOCTL range: 0x70 - 0x7F (16)
 */
#define EM_WAIT_INIT                         _IOW(FZM_IOCTL_ID, 0x70, struct em_wait_ioctl_init) /*init*/
#define EM_WAIT_WAIT_FOR                     _IOW(FZM_IOCTL_ID, 0x71, struct em_wait_ioctl_wait_for) /*wait for interrupt or timeout*/

#endif /* FZM_IOCTLS_H */
