// *****************************************************************************
// PROJECT NAME     :   FZM
// Component name   :   iwfD.h
// Filename         :   
// Author           :   Abhilash.B
// Date created     :   19.02.2013
// Date changed     :   -
// -----------------------------------------------------------------------------
// Description      :   
//
// -----------------------------------------------------------------------------
// Revision history :
// date       | author         | comments
// ---------- + -------------- +----------------------------------------------------
// 19.02.2013 | Abhilash.B     | Initial version
// 20.03.2013 | Shwethambari C | Added extension header part
// 21.03.2013 | Maniraj Patri  | Added Error indication functionality
//            |                |
// *****************************************************************************
// Copyright (c) Nokia 2013
// All rights reserved.
// Licensed material - property of Nokia
// *****************************************************************************
#ifndef _iwfd_header_
#define _iwfd_header_

#define _IWFD_ENABLE_DSP_SIMULATOR_
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <asm/byteorder.h>
/* for header struct */
#include <linux/if_ether.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include <linux/if_vlan.h>

#include <linux/in.h>
#include <linux/inet.h>
#include "ip_secfg_IWF_VTC.h"
#include "l2switch_cfg.h"

/* common IPV4 and IPV6 marcos */
#define IPV6HDR_LEN sizeof(struct ipv6hdr)
#define IPV4HDR_LEN sizeof(struct iphdr)

#define IPV6HDR_P_UDPHDR_LEN (sizeof(struct ipv6hdr) + sizeof(struct udphdr))
#define IPV4HDR_P_UDPHDR_LEN (sizeof(struct iphdr) + sizeof(struct udphdr))

#define UDPHDR_LEN sizeof(struct udphdr)

/* macros common to IWF module */
#define _IWF_MODULE_NAME "NSN IWF"
#define _IWF_MAX_FRAME_LEN 1500

#define _IWF_SUCCESS_       1
#define _IWF_FAILURE_       -1

/* macros specific to IWF Daemon */
#define _IWFD_DL_UDP_PORT_ 2152


#define _BFD_SINGLE_HOP_UDP_PORT_ 3784
#define _BFD_MULTI_HOP_UDP_PORT_  4784
#define _BFD_MIN_UDP_PORT_        49152
#define _BFD_MAX_UDP_PORT_        65535


#ifdef  _IWFD_ENABLE_DSP_SIMULATOR_
   #define _IWFD_UL_UDP_PORT_ 30000
#else
   #define _IWFD_UL_UDP_PORT_ _IWFD_DL_UDP_PORT_
#endif

/* path supervision */
#define _IWF_GTP_ECHO_REQ                       1
#define _IWF_GTP_ECHO_RESP                      2
#define _IWF_GTP_ERR_IND                        26
#define _IWF_GTP_EXT_HDR_NOTI                   31
#define _IWF_GTP_MGMT_MSG_DSCP                  46

#define _IWF_GTP_ECHO_RESP_LEN                  6  // UC-5: GTPu length in byte
#define _IWF_GTP_IE_TYPE_4                      14 // IE Recovery
#define _IWF_MAX_SGW_IP_ADDRESSES               MAX_SGW_IP_ADDRESSES
#define _IWF_GTP_SUPPOTERD_EXT_HDR              31
#define _IWF_GTP_EXT_HDR_NOTI_LEN               8
#define GTP_IE_TYPE_3                           141
#define GTP_IE_TYPE_3_LEN                       2
#define _IWF_GTP_EXT_HDR_UDP                    0x40
#define _IWF_GTP_EXT_HDR_PDCP                   0xC0
#define _IWF_GTP_EXT_HDR_MSG_TYPE_UDP           26
#define _IWF_GTP_EXT_HDR_MSG_TYPE_PDCP          255
/* SIC Message relevant details */
#define _IWF_SIC_VERSION                     1       /**< SIC egress message header value: Version (4 bits) */
#define _IWF_SIC_RESERVED                    0       /**< SIC egress message header value: Reserved (12 bits) */
#define _IWF_SIC_HEADER_FLAGS_1              0       /**< SIC egress message header value: Header Flags 1 (8 bits) */
#define _IWF_SIC_HEADER_FLAGS_2              8       /**< SIC egress message header value: Header Flags 2 (8 bits) */


#define TRSW_UPLANE_RECEIVE_DATA_MSG           0x3424

#define _IPV6_MULTICAST                    0xff
/************ MANI ************************/

#define FLAG_GTP_ERROR_IND 0
#define GTPU_MSG_ERR_IND 26 //replace with _IWF_GTP_ERR_IND
#define GTP_ERROR_IND_GTP_LEN 20
#define GTPU_MGMT_EXT_HDR_UDPPORT 64   //_IWF_GTP_EXT_HDR_UDP
#define LTM_GTPU_PRT 2152      //_IWFD_DL_UDP_PORT_
#define GTP_IE_TYPE_1 16
#define GTP_IE_TYPE_2 133
/*IP Address length bytes: Spec 3GPP TS 23.003 section 5.1 GSN Address*/
#define GTP_IE_TYPE_2_LEN_IPV4 4
#define GTP_IE_TYPE_2_LEN_IPV6 0x10
#define FLAG_GTP_EXT_HDR_NOTI  1
#define GTPU_MSG_EXT_HDR_NOTI 31
#define GTP_EXT_HDR_NOTI_GTP_LEN 8
#define GTPU_MGMT_EXT_HDR_PDCP_NUM 0xC0 // _IWF_GTP_EXT_HDR_PDCP
#define FLAG_GTP_ECHO_RESP 2 //_IWF_GTP_ECHO_RESP
#define GTPU_MSG_ECHO_RES 2  //_IWF_GTP_ECHO_RESP
#define GTP_ECHO_RESP_GTP_LEN 6  //_IWF_GTP_ECHO_RESP_LEN
#define GTP_IE_TYPE_4 14
#define GTP_MGMT_HDR_LEN_WO_OPTS_AND_EXT        8
#define GTP_MGMT_HDR_LEN_WITH_OPTS_WO_EXT       12
#define GTP_MGMT_HDR_LEN_WITH_OPTS_AND_EXT      16


//typedef unsigned long uint64_t;



typedef struct fzm_iwf_ip_in_addr
{
   uint32_t  s_addr;
} s_fzm_iwf_ip_in_addr;

/* ******************************************** */
#if 0 //TODO : another enum already exist, refer below
typedef enum 
{
    CLASSIFY_DEFAULT = 0,
    CLASSIFY_DROP,
    CLASSIFY_ARP,
    CLASSIFY_HOST,
    CLASSIFY_ATMGW,
    CLASSIFY_WESTPORT,
    CLASSIFY_WESTPORT_FROM_OTHER_MODULE,
    CLASSIFY_TRANSPORT,
    CLASSIFY_FTI,
    CLASSIFY_OTHER_MODULE,
    CLASSIFY_TRANSMIT,
    CLASSIFY_IPSEC,
    CLASSIFY_REASSEMBLY,
    CLASSIFY_SRIO_DOWNLINK,
    CLASSIFY_SRIO_UPLINK,
    CLASSIFY_IP_FWD,
    CLASSIFY_CONTINUE,
    CLASSIFY_SRIO_DOWNLINK_SMALL,
    CLASSIFY_ABISSE
} classify_result_t;
#endif

/* iwf error values */
typedef enum
{
    _IWF_FAILED = -1,
    _IWF_SUCCESS
}_IWF_RET_VAL;

/* classify packet */
typedef enum
{
    
    _IWF_PKT_CLASSIFIER_INVALID_ARG = -1,
    _IWF_DROP_PKT,
    _IWF_FWD_PKT_TO_TUPU,
    _IWF_FWD_PKT_TO_SGW,
    _IWF_FWD_ERR_IND_TO_TUPU,
    _IWF_SEND_ECHO_RESPONSE_TO_SGW,
    _IWF_FWD_ECHO_RESPONSE_TO_TUPU,
    _IWF_FWD_ECHO_REQUEST_TO_SGW,
    _IWF_SEND_SUPPORTED_EXT_HDR_TO_SGW,
    _IWF_SEND_ERR_IND_TO_SGW
}_IWF_PKT_CLASSIFIER_RESULT;


typedef enum
{
     /* NOTE : enums should be assigned values expliciltly  */
    _IWF_EXT_HDR_PRESENT=1,
    _IWF_EXT_HDR_NOT_PRESENT=0
}_IWF_COMMON;

typedef enum 
{
    _IWF_AVAILABLE_IN_DL_TABLE,
    _IWF_NOT_AVAILABLE_IN_DL_TABLE,
    _IWF_AVAILABLE_IN_UL_TABLE,
    _IWF_NOT_AVAILABLE_IN_UL_TABLE
}_IWF_TABLE_LOOKUP_RESULT;

enum 
{
    _IWF_ERR_HDR,
    _IWF_NOHDR,
    _IWF_EXT_HDR,
    _IWF_UL_HDR
};

typedef struct 
{
    /* WORD 2 */
    uint8_t ext_hdr_len;        // total length of extension header, for UDP == 1
    uint8_t ext_hdr_udp_prt_1;  // for UDP_PRT == GTPU
    uint8_t ext_hdr_udp_prt_0;  // for UDP_PRT == GTPU
    uint8_t nxt_ext_hdr_type;   // only 1 ext hdr is allowed => 0x0

    /* WORD 3 + 4 */
    uint8_t type_1;            // value dec16 for p_TEID, IE-1
    uint8_t p_teid_b3;
    uint8_t p_teid_b2;
    uint8_t p_teid_b1;
    uint8_t p_teid_b0;

    uint8_t type_2;            // value dec133 for p_IP, IE-2
    uint8_t length_1;          // value dec4 0r 0x0004 to indicate IPv4, 0x0050 to IPV6
    uint8_t length_0;          // value dec4 0r 0x0004 to indicate IPv4, 0x0050 to IPV6

    /* WORD 5 */
    uint32_t   p_ip;
    uint32_t   p_ip1;
    uint32_t   p_ip2;
    uint32_t   p_ip3;

} gtp_error_ind_ext_hdr_ie_data_t;

typedef struct
{
    /* WORD 3 + 4 */
    uint8_t type_1;            // value dec16 for p_TEID, IE-1
    uint8_t p_teid_b3;
    uint8_t p_teid_b2;
    uint8_t p_teid_b1;
    uint8_t p_teid_b0;

    uint8_t type_2;            // value dec133 for p_IP, IE-2
    uint8_t length_1;          // value dec4 0r 0x0004 to indicate IPv4
    uint8_t length_0;          // value dec4 0r 0x0004 to indicate IPv4

    /* WORD 5 */
    uint32_t   p_ip;
    uint32_t   p_ip1;
    uint32_t   p_ip2;
    uint32_t   p_ip3;
 
}gtp_error_ind_no_udp_ie_data_t;

typedef struct 
{
    /* WORD 2 */
    uint8_t ext_hdr_len;        // total length of extension header, for UDP == 1
    uint8_t ext_hdr_udp_prt_1;  // for UDP_PRT == GTPU
    uint8_t ext_hdr_udp_prt_0;  // for UDP_PRT == GTPU
    uint8_t nxt_ext_hdr_type;   // only 1 ext hdr is allowed => 0x0

    /* WORD 3 + 4 */
    uint8_t type_1;            // value dec16 for p_TEID, IE-1
    uint8_t p_teid_b3;
    uint8_t p_teid_b2;
    uint8_t p_teid_b1;
    uint8_t p_teid_b0;

    uint8_t type_2;            // value dec133 for p_IP, IE-2
    uint8_t length_1;          // value dec4 0r 0x0004 to indicate IPv4
    uint8_t length_0;          // value dec4 0r 0x0004 to indicate IPv4

    /* WORD 5 */
    uint32_t   p_ip;

} gtp_error_ind_ext_hdr_ie_ipv4_data_t;

typedef struct
{
    /* WORD 2 */

    uint8_t  type_3;          // value dec141 for ExtHdrType list, IE-3
    uint8_t  length;          // value dec1 indicates that we have only 1 Ext Hdr Type here
    uint8_t  ext_hdr_type_1;  // we should only have one ext_hdr_type !
    uint8_t  ext_hdr_type_2;
} gtp_ext_hdr_noti_ie_data_t;

typedef struct 
{
    uint8_t  tlen;              // total length of extension header
    //uint8_t  * content;           // extension header content
    //uint8_t  nxt_ext_hdr;     // Next extension header
} gtp_nxt_ext_hdr_t;

typedef struct
{
    uint8_t  type;   //value dec:14
    uint8_t  restart_counter;
} gtp_recovery_counter_ie_data_t;

typedef struct iwfGtpuHdrFlags
{
#ifdef __LITTLE_ENDIAN

    uint8_t  npdu_num_present:1;
    uint8_t  seq_nr_present:1;
    uint8_t  ext_hdr_present:1;
    uint8_t  reserved:1;
    uint8_t  proto_type:1;
    uint8_t  version:3;
#else
    uint8_t  version:3;
    uint8_t  proto_type:1;
    uint8_t  reserved:1;
    uint8_t  ext_hdr_present:1;
    uint8_t  seq_nr_present:1;
    uint8_t  npdu_num_present:1;
#endif

}__attribute__ ((packed)) iwf_gtp_hdr_flags_t;

typedef struct iwfGtpuHeader
{
    iwf_gtp_hdr_flags_t flags;
    uint8_t             msg_type;
    uint16_t            total_len;
    uint32_t            teid;
 
}__attribute__ ((packed)) iwf_gtp_hdr_t;

#if 0
typedef struct iwfGtpuPacket
{
    iwf_gtp_hdr_t      hdr; 
    uint8_t             payload[1];
}__attribute__ ((packed)) iwf_gtp_pkt_t;
#endif
// unions for different IEs & next extension header
typedef union iwfGtpuExtensionHeader
{  
    gtp_error_ind_ext_hdr_ie_data_t gtp_error_ind_ext_hdr_ie_data;
    gtp_ext_hdr_noti_ie_data_t  gtp_ext_hdr_noti_ie_data;
    gtp_nxt_ext_hdr_t           gtp_nxt_ext_hdr;
    gtp_recovery_counter_ie_data_t  gtp_recovery_ie_data;
}__attribute__ ((packed)) iwf_gtp_extension_header_u;

typedef struct iwfGtpuHeaderOptionalFields
{
    uint16_t seq_num;
    uint8_t  npdu_num;
    uint8_t  nxt_exthdr_type;
}__attribute__ ((packed)) iwf_gtp_hdr_optional_t;


typedef struct iwfGtpuPayload
{
    iwf_gtp_hdr_optional_t optionalField;
    iwf_gtp_extension_header_u   extHdr;

}__attribute__ ((packed)) iwf_gtp_payload_t;

typedef struct iwfSicVersion  
{
#ifdef __LITTLE_ENDIAN
    uint16_t reserved:12;
    uint16_t version:4;
#else
    uint16_t version:4;
    uint16_t reserved:12;
#endif
}iwf_sic_version_t;

typedef union iwfSicad
{
    uint32_t sicad_32;
    struct
    {

      #ifdef __LITTLE_ENDIAN
         uint16_t cpid;
         uint16_t nid;
      #else
         uint16_t nid;
         uint16_t cpid;
      #endif

    };

}__attribute__ ((packed)) iwf_sicad_t;


typedef struct iwfSicMessageHeader 
{


#ifdef __LITTLE_ENDIAN
    uint16_t mtn;
    iwf_sic_version_t hdr;

#else
    iwf_sic_version_t hdr;
    uint16_t mtn;
#endif
#ifdef __LITTLE_ENDIAN
    uint16_t rx_cpid;
    uint16_t rx_nid;
#else
    uint16_t rx_nid;
    uint16_t rx_cpid;
#endif
#ifdef __LITTLE_ENDIAN
    uint16_t tx_cpid;
    uint16_t tx_nid;
#else
    uint16_t tx_nid;
    uint16_t tx_cpid;
#endif

#ifdef __LITTLE_ENDIAN
    uint8_t  hdr_flags_2;
    uint8_t  hdr_flags_1;
    uint16_t mlen;
#else
    uint16_t mlen;
    uint8_t  hdr_flags_1;
    uint8_t  hdr_flags_2;

#endif

}__attribute__ ((packed)) iwf_sic_message_hdr_t;

typedef struct iwfGtpuMgmtPacket
{
   iwf_gtp_hdr_t         gtpuHdr;
   iwf_gtp_payload_t     payload;
}iwf_gtp_mgmt_pkt_t;

/* function prototype */
int  iwfSendSicMessage(iwf_sic_message_hdr_t *sicHdr);
int iwfFwdEchoResponseToTupu(uint8_t *data, int iDataLen);
int iwfSendEchoResponseToSgw(struct sk_buff * skb);
int iwfSendSuppExthdrToSgw(struct sk_buff * skb);
int iwfSendErrorIndMsgToSgw(struct sk_buff *skb);
_IWF_PKT_CLASSIFIER_RESULT  classifyOperationOnDLPkt(uint8_t *data,int iDataLen,
                            _IWF_TABLE_LOOKUP_RESULT lookupResult,sicp_data_t *outSicpConfigData, bool isFP,struct sk_buff *skb);

bool checkForValidProtocol(struct iphdr *ip_header);
bool checkForV4ErrorPackets(struct sk_buff *skb);
int checkHdrType(uint16_t hdr);
bool isValidProtoAndExtnHdr(struct sk_buff *skb);
bool checkForV6ErrorPackets(struct sk_buff *skb);
bool isValidateIpv4HeaderChecksum(struct sk_buff *skb);
bool isValidateIcmpv4Checksum(struct sk_buff *skb);
bool isValidateIcmpv6Checksum(struct sk_buff *skb);
bool isValidateUdp6Checksum(struct sk_buff *skb);
void increamentDropCounter(struct sk_buff *skb,uint8_t version);

void vfilter_hook_register(void);
unsigned int vfilter_br_nf_local(unsigned int hooknum,struct sk_buff *skb,const struct net_device *in,
             const struct net_device *out,int (*okfn)(struct sk_buff*));
unsigned int vfilter_br_nf_local_out(unsigned int hooknum,struct sk_buff *skb,const struct net_device *in,
             const struct net_device *out,int (*okfn)(struct sk_buff*));
#endif
