/***********************************************************************
 * PROJECT NAME       : FZM
 * Component name     : SICP API
 * File name          : sicp_iwf_downlink.c
 * Author             :
 * Date created       : 
 * Date modified last : 
 * ---------------------------------------------------------------------
 * Description        :
 *
 ***********************************************************************
 * Copyright (c) Nokia 2010 - 2012
 * All rights reserved.
 * Licensed material - property of Nokia
 ***********************************************************************
 * Revision history (latest on top)
 *
 * Date        | Who | Description
 * ---------------------------------------------------------------------
 ***********************************************************************/
/** @file srio_iwf_downlink.c */

#include "iwf_table_creation.h"

extern shared_hash_table_t  * shared_hash_table;
extern sicp_data_t        * teid0_sicp_data;
extern logfile_block_names_ptr_t * pSeLogfileBlockNamePointer;
extern unsigned int iwf_log_level;

uint32_t set_teid_sicp_connection (uint32_t teid_dl, uint16_t rx_nid,  uint16_t tx_nid,
                                   uint16_t rx_cpid, uint16_t tx_cpid, uint16_t mtype)
{
    uint16_t                  hash        = 0;
    teid_sicp_hash_bucket_t * bucket      = NULL;
    teid_sicp_hash_bucket_t * head_bucket = NULL;
    unsigned int              i           = 0;

    // TEID0 SICP DATA table
    if (NULL == teid0_sicp_data)
    {
            //// Memory to be malloced.
	    teid0_sicp_data = (sicp_data_t *)kmalloc (sizeof(sicp_data_t), GFP_KERNEL);
                                                            
            if (teid0_sicp_data == NULL)
            {
                if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
                { 
                    if(0 != printk_ratelimit()) 
                    {
                        printk(" RDRD ERR set_teid_sicp_connection teid0_sicp_data = NULL\n");
                    }
                }
                return (1);
            }
            
            if (NULL != pSeLogfileBlockNamePointer)
            {
                /* store table pointer for the logfile handling */
                pSeLogfileBlockNamePointer->teid0_sicp_data_table_ptr = (uint32_t)teid0_sicp_data;
            }
    }
    

    if (!((rx_nid) && (tx_nid) && (rx_cpid) && (tx_cpid)))
    {
        if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
        { 
            if(0 != printk_ratelimit()) 
            {
                printk(" RDRD ERR set_teid_sicp_connection rx_nid =%u,tx_nid=%u,rx_cpid=%u,tx_cpid=%u\n",rx_nid,tx_nid,rx_cpid,tx_cpid);
            }
        }

        return VTC_ERR;
    }
    if (teid_dl == 0)
    {
        teid0_sicp_data->rx_nid  = rx_nid;
        teid0_sicp_data->tx_nid  = tx_nid;
        teid0_sicp_data->rx_cpid = rx_cpid;
        teid0_sicp_data->tx_cpid = tx_cpid;
        teid0_sicp_data->mtype   = mtype;

        if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
        { 
            if(0 != printk_ratelimit()) 
            {
                printk(" RDRD OK set_teid_sicp_connection rx_nid =%u,tx_nid=%u,rx_cpid=%u,tx_cpid=%u mtype=%u\n",rx_nid,tx_nid,rx_cpid,tx_cpid,mtype);
            }
        }

        return VTC_OK;
    }

    hash = hash_table_do_hash (teid_dl);
     
    head_bucket = &shared_hash_table[hash].teid_sicp_table;
    bucket = head_bucket;

    while (bucket)
    {
    
        for (i = 0; i < TEID_SICP_HASH_ENTRIES_PER_BUCKET; i++)
        {
            teid_sicp_hash_entry_t *entry = &(bucket->entries[i]);

	     
            if (entry->teid == teid_dl)
            {
                if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
                { 
                    if(0 != printk_ratelimit()) 
                    {
                        printk(" RDRD ERR duplicate teid %u,%u\n",entry->teid,teid_dl);
                    }
                }
                return ERR_DUPLICATE_TEID;
            }
            if (entry->teid == 0)
            {
                // found free entry
                entry->teid = teid_dl;
                entry->sicp_data.rx_nid  = rx_nid;
                entry->sicp_data.tx_nid  = tx_nid;
                entry->sicp_data.rx_cpid = rx_cpid;
                entry->sicp_data.tx_cpid = tx_cpid;
                entry->sicp_data.mtype   = mtype;
                bucket->entries_used++;
                return VTC_OK;
            }
        }
        bucket = bucket->next;
    }


    bucket = head_bucket;
    while (bucket->next)
        bucket = bucket->next;


    // no free entry found in any of the linked buckets, create a new one
    //
    // Malloc to be done here.
    bucket->next = (teid_sicp_hash_bucket_t *) kmalloc(sizeof (teid_sicp_hash_bucket_t), GFP_KERNEL);
    
    if (bucket->next == NULL)
    {
        if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
        { 
            if(0 != printk_ratelimit()) 
            {
                printk(" RDRD ERR set_teid_sicp_connection NULL bucket\n");
            }
        }
       return VTC_ERR;
    }

    memset(bucket->next, 0, sizeof(teid_sicp_hash_bucket_t));

    {
        teid_sicp_hash_entry_t *entry = &(bucket->next->entries[0]);
        entry->teid              = teid_dl;
        entry->sicp_data.rx_nid  = rx_nid;
        entry->sicp_data.tx_nid  = tx_nid;
        entry->sicp_data.rx_cpid = rx_cpid;
        entry->sicp_data.tx_cpid = tx_cpid;
        entry->sicp_data.mtype   = mtype;
        bucket->next->entries_used++;
        
        if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
        { 
            if(0 != printk_ratelimit()) 
            {
                printk("RDRD OK teid_dl %u rx_nid =%u,tx_nid=%u,rx_cpid=%u,tx_cpid=%u mtype=%u, entries %llu\n",teid_dl,rx_nid,tx_nid,rx_cpid,tx_cpid,mtype,(unsigned long long int )bucket->next->entries_used);
            }
        }
        return VTC_OK;
    }
}

uint32_t ip_se_teid_sicp_set (sIpSicpApiMsg *msg)
{
    // Down-link / ingress processing
    ip_sicp_api_t * cfg       = (ip_sicp_api_t *) getCfgMsgPayload(msg);
    sicp_data_t   * conf_data = (sicp_data_t *)  &cfg->u.teid_sicp_data;
    uint32_t retVal           = VTC_OK;

    // function call to write entry into the table
    retVal = set_teid_sicp_connection(cfg->up.teid_dl, conf_data->rx_nid, conf_data->tx_nid, conf_data->rx_cpid,
                                      conf_data->tx_cpid, conf_data->mtype);

    return retVal;
}

uint32_t mod_teid_sicp_connection (uint32_t teid_dl, uint16_t rx_nid, uint16_t rx_cpid)
{
    uint32_t                  hash   = 0;
    
    
    teid_sicp_hash_bucket_t * bucket = NULL;
    unsigned int              i      = 0;

    if (! rx_cpid )
    {
        if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
        { 
            if(0 != printk_ratelimit()) 
            {
                printk(" RDRD ERR mod_teid_sicp_connection rx_cpid =0\n");
            }
        }
        return VTC_ERR;
    }
    
    if (teid_dl == 0)
    {
        teid0_sicp_data->rx_nid = rx_nid;  //rx_nid 0 is valid for 72007ESPE01
        if (rx_cpid)
        {
            teid0_sicp_data->rx_cpid = rx_cpid;
        }
        return VTC_OK;
    }

    hash = hash_table_do_hash (teid_dl);

    bucket = &shared_hash_table[hash].teid_sicp_table;


    while (bucket)
    {
        for (i = 0; i < TEID_SICP_HASH_ENTRIES_PER_BUCKET; i++)
        {
            teid_sicp_hash_entry_t *entry = &(bucket->entries[i]);
            if (entry->teid == teid_dl)
            {
                // entry found, update it
                entry->sicp_data.rx_nid  = rx_nid; 
                if ( rx_cpid )
                    entry->sicp_data.rx_cpid = rx_cpid;
                return VTC_OK;
            }
        }
        bucket = bucket->next;
    }

    if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
    { 
        if(0 != printk_ratelimit()) 
        {
            printk(" RDRD ERR mod_teid_sicp_connection bucket is NULL\n");
        }
    }
 
   return VTC_ERR;
}

uint32_t ip_se_teid_sicp_mod (sIpSicpApiMsg *msg)
{
    ip_sicp_api_t * cfg       = (ip_sicp_api_t *) getCfgMsgPayload(msg);
    sicp_data_t   * conf_data = (sicp_data_t *)  &cfg->u.teid_sicp_data;
    uint32_t        retVal    = VTC_OK;

    retVal = mod_teid_sicp_connection(cfg->up.teid_dl, conf_data->rx_nid, conf_data->rx_cpid);

    return retVal;
}
//NOTE: Debug prints in function will hog Debug log for DL traffic
// Please Note this before adding any debug print here
sicp_data_t * get_teid_sicp_connection (uint32_t teid_dl)
{
    uint32_t                  hash   = 0;
    teid_sicp_hash_bucket_t * bucket = NULL;
    unsigned int              i      = 0;

    if (teid_dl == 0)
    {
      //printk(" RDRD ERR get_teid_sicp_connection teid_dl is NULL\n");   
     return NULL;
    }
    hash = hash_table_do_hash (teid_dl);

    bucket = &shared_hash_table[hash].teid_sicp_table;

    while (bucket)
    {
        for (i = 0; i < TEID_SICP_HASH_ENTRIES_PER_BUCKET; i++)
        {
            teid_sicp_hash_entry_t *entry = &(bucket->entries[i]);
            if (entry->teid == teid_dl)
            {
                // entry found, return it
                return &(entry->sicp_data);
            }
        }
        bucket = bucket->next;
    }
    //printk(" RDRD ERR get_teid_sicp_connection return NULL bucket\n");
   return NULL;
}

uint32_t ip_se_teid_sicp_mod_ccid(sIpSicpApiMsg *msg)
{
    ip_sicp_api_t * cfg       = (ip_sicp_api_t *) getCfgMsgPayload(msg);
    sicp_data_t   * conf_data = (sicp_data_t *)  &cfg->u.teid_sicp_data;
    uint32_t        retVal    = VTC_OK;

    retVal = mod_teid_sicp_connection_ccid(cfg->up.teid_dl, conf_data->tx_cpid);

    return retVal;
}


uint32_t ip_se_teid_sicp_del (sIpSicpApiMsg *msg)
{
    // Down-link / ingress processing
    ip_sicp_api_t    * cfg       = (ip_sicp_api_t *)getCfgMsgPayload(msg);
    uint32_t           teid_dl   = cfg->up.teid_dl;
    uint32_t           retVal    = VTC_OK;

    retVal = del_teid_sicp_connection(teid_dl);

    return retVal;
}

uint32_t ip_se_teid_sicp_get (sIpSicpApiMsg *msg)
{
    /* Down-link / ingress processing */
    ip_sicp_api_t * cfg       = (ip_sicp_api_t *)getCfgMsgPayload(msg);
    sicp_data_t   * conf_data = get_teid_sicp_connection(cfg->up.teid_dl);
    uint32_t        retVal    = VTC_OK;

    if (conf_data != NULL)
    {
        /* Finally read values should be given back towards host, not only printed! */
        memcpy(&(cfg->u.teid_sicp_data), conf_data, sizeof(sicp_data_t));

        conf_data->rx_nid  = conf_data->rx_nid;
        conf_data->tx_nid  = conf_data->tx_nid;
        conf_data->rx_cpid = conf_data->rx_cpid;
        conf_data->tx_cpid = conf_data->tx_cpid;
        conf_data->mtype   = conf_data->mtype;
    }
    else
    {
        if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
        { 
            if(0 != printk_ratelimit()) 
            {
                printk("RDRD ip_se_teid_sicp_get conf_data is NULL\n");
            }
        }
       retVal = VTC_ERR;
    }

    return retVal;
}


uint32_t del_teid_sicp_connection (uint32_t teid_dl)
{
    uint32_t                  hash        = 0;
    unsigned int              i           = 0;
    teid_sicp_hash_bucket_t * head_bucket = NULL;
    teid_sicp_hash_bucket_t * bucket      = NULL;

    if (teid_dl == 0)
    {
        if(teid0_sicp_data){
            memset(teid0_sicp_data, 0, sizeof(sicp_data_t));
            return VTC_OK;
        }
        else
	{
        if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
        { 
            if(0 != printk_ratelimit()) 
            {
                printk("RDRD del_teid_sicp_connection teid0_sicp_data is NULL \n"); 
            }
        }
           return VTC_ERR;
        }
    }

    hash = hash_table_do_hash (teid_dl);

    head_bucket = &shared_hash_table[hash].teid_sicp_table;
    bucket = head_bucket;

    while (bucket)
    {
        for (i = 0; i < TEID_SICP_HASH_ENTRIES_PER_BUCKET; i++)
        {
            teid_sicp_hash_entry_t *entry = &(bucket->entries[i]);
            if (entry->teid == teid_dl)
            {
                // entry found, delete it
                memset(entry, 0, sizeof(teid_sicp_hash_entry_t));
                bucket->entries_used--;

                if ((bucket->entries_used == 0) && (bucket != head_bucket))
                {
                    // bucket is empty, if it's not the first bucket, delete it
                    teid_sicp_hash_bucket_t *temp = head_bucket;
                    while (temp->next != bucket)
                    {
                        temp = temp->next;
                    }
                    temp->next = bucket->next;
                    memset (bucket, 0, sizeof (teid_sicp_hash_bucket_t));
		    // Free the entry 
		    kfree (bucket);
                }
                return VTC_OK;
            }
        }
        bucket = bucket->next;
    }

    if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
    { 
        if(0 != printk_ratelimit()) 
        {
            printk(" RDRD ERR del_teid_sicp_connection teid_dl %u\n",teid_dl);
        }
    }
   return VTC_ERR;
}


uint32_t mod_teid_sicp_connection_ccid(uint32_t teid_dl, uint16_t tx_cpid)
{
    uint32_t                  hash   = 0;
    teid_sicp_hash_bucket_t * bucket = NULL;
    unsigned int              i      = 0;
    if (! tx_cpid )
    {
       return VTC_ERR;
    }
    if (teid_dl == 0)
    {
        if (tx_cpid)
        {
            teid0_sicp_data->tx_cpid = tx_cpid;
        }
        return VTC_OK;
    }

    hash = hash_table_do_hash (teid_dl);

    bucket = &shared_hash_table[hash].teid_sicp_table;

    while (bucket)
    {
        for (i = 0; i < TEID_SICP_HASH_ENTRIES_PER_BUCKET; i++) 
        {
            teid_sicp_hash_entry_t *entry = &(bucket->entries[i]);
            if (entry->teid == teid_dl)
            {
                // entry found, update it
                if ( tx_cpid )
                {
                    entry->sicp_data.tx_cpid = tx_cpid;
                }
                return VTC_OK;
            }
        }
        bucket = bucket->next;
    }
   
    if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
    { 
        if(0 != printk_ratelimit()) 
        {
            printk(" RDRD ERR mod_teid_sicp_connection_ccid returning error\n");
        }
    }
 
  return VTC_ERR;
}
uint32_t ip_se_downlink_proc_dump (struct seq_file *m)
{
   uint16_t num_connections = 0;
   uint16_t num_primary_buckets = 0;
   uint16_t num_secondary_buckets = 0;
   uint32_t i;
   int retVal = 0;


	seq_printf(m,"\nLTE TEID-hash table for Downlink IWF:\n");
	seq_printf(m,"=======================================\n");
	seq_printf(m,"Configured IP->SRIO connections:\n");
	seq_printf(m,"=====\n\n");
	seq_printf(m,"TEID      \tRX_NID\tTX_NID\tRX_CPID\tTX_CPID\tMTYPE\n");
	seq_printf(m,"====      \t======\t======\t=======\t=======\t======\n");

   if (NULL != shared_hash_table)
   {
        /* only if valid pointer, else we would get a seg fault */

       for (i = 0; i < TEID_SICP_HASH_BUCKETS; i++)
       {
          teid_sicp_hash_bucket_t *head_bucket = &shared_hash_table[i].teid_sicp_table;
          teid_sicp_hash_bucket_t *bucket = head_bucket;

           while (bucket)
           {
              unsigned int j;
              unsigned int found = 0;
              if (bucket != head_bucket)
              {
                 num_secondary_buckets++;
              }
              for (j = 0; j < TEID_SICP_HASH_ENTRIES_PER_BUCKET; j++)
              {
                teid_sicp_hash_entry_t *entry = &(bucket->entries[j]);
                if (entry->teid)
                {
						seq_printf(m,"0x%08x\t0x%04x\t0x%04x\t 0x%04x\t 0x%04x\t0x%04x\n",
                             entry->teid, entry->sicp_data.rx_nid, entry->sicp_data.tx_nid,
                             entry->sicp_data.rx_cpid, entry->sicp_data.tx_cpid, entry->sicp_data.mtype);
                   num_connections++;
                   if (!found)
                   {
                      if (bucket == head_bucket)
                      {
                          num_primary_buckets++;
                      }
                      found = 1;
                    }
                  }
                }
                bucket = bucket->next;
             }
         }
         retVal = VTC_OK;
      }
      else
      {
		seq_printf(m,"LTE TEID-hash table for Downlink IWF table pointer = NULL !!!\n");
		seq_printf(m,"Downlink Dump Failed\n\n");
         retVal = VTC_ERR;
      }
     return retVal;
}

